@@ -146,7 +146,6 @@ class UMNDirHandler(DirHandler):
                 setattr(old, field, getattr(new, field))
 
         for field in new.geteadict().keys():
-            print "Handling ea", field
             old.setea(field, new.getea(field))
 
     def processLinkFile(self, filename, capfilepath = None):
@@ -244,7 +243,7 @@ class UMNDirHandler(DirHandler):
         if done['path']:
             if entry.getneedsabspath() and \
                    entry.gethost() == None and entry.getport() == None:
-                entry.setselector(self.selectorbase + "/" + entry.getselector())
+                entry.setselector(os.path.normpath(self.selectorbase + "/" + entry.getselector()))
             return (nextstep, entry)
         return (nextstep, None)
 
