@@ -22,7 +22,7 @@ from ConfigParser import ConfigParser
 
 # Import lots of stuff so it's here before chrooting.
 import socket, os, sys, SocketServer, re, stat, os.path, UserDict, xreadlines, tempfile
-import time
+import time, atexit
 
 from pygopherd import handlers, protocols, GopherExceptions, logger
 from pygopherd.protocols import *
@@ -147,14 +147,30 @@ def initsecurity(config):
         os.setreuid(idsetuid, idsetuid)
         logger.log("Switched to uid %d" % idsetuid)
 
+def initconditionaldetach(config):
+    if config.getboolean("pygopherd", "detach"):
+        pid = os.fork()
+        if pid:
+            logger.log("Parent process detaching; child is %d" % pid)
+            sys.exit(0)
+
+def initpidfile(config):
+    if config.has_option("pygopherd", "pidfile"):
+        pidfile = config.get("pygopherd", "pidfile")
+        fd = open(pidfile, "wt")
+        fd.write("%d\n" % os.getpid())
+        fd.close()
+
 def initeverything(conffile):
     config = initconffile(conffile)
     initlogger(config, conffile)
     initexceptions(config)
     initmimetypes(config)
     s = getserverobject(config)
+    initconditionaldetach(config)
+    initpidfile(config)
     initsecurity(config)
-    
-    logger.log("Root is '%s'" % config.get("pygopherd", "root"))
+
+    logger.log("Running.  Root is '%s'" % config.get("pygopherd", "root"))
     return s
 
