@@ -66,25 +66,43 @@ class GopherPlusProtocol(GopherProtocol):
             else:
                 handler.prepare()
                 self.wfile.write("+" + str(self.entry.getsize(-2)) + "\r\n")
-                handler.write(self.wfile)
+                if handler.isdir():
+                    self.writedir(self.entry, handler.getdirlist())
+                else:
+                    handler.write(self.wfile)
         except GopherExceptions.FileNotFound, e:
             self.filenotfound(str(e))
         except IOError, e:
             GopherExceptions.log(e, self, None)
             self.filenotfound(e[1])
 
-    def getsupportedblocknames(self):
-        return ['+INFO', '+ADMIN', '+VIEWS']
+    def getsupportedblocknames(self, entry):
+        # Return the always-supported values PLUS any extra ones for
+        # this particular entry.
+        return ['+INFO', '+ADMIN', '+VIEWS'] + \
+               ['+' + x for x in entry.geteadict().keys()]
 
     def getallblocks(self, entry):
         retstr = ''
-        for block in self.getsupportedblocknames():
+        for block in self.getsupportedblocknames(entry):
             retstr += self.getblock(block, entry)
         return retstr
 
     def getblock(self, block, entry):
+        # If the entry has the block in its eadict, return that.
+        # Otherwise, do our own thing.
         # Incoming block: +VIEWS
         blockname = block[1:].lower()
+
+        if entry.geteadict().has_key(blockname.upper()):
+            return "+" + blockname.upper() + ":\r\n" + \
+                   ''.join(
+                           [" " + x + "\r\n" for x in \
+                           entry.getea(blockname.upper()).splitlines()]
+                   )
+
+        # Not in there -- look up a custom function.
+        
         # Name: views
         funcname = "get" + blockname + "block"
         # Funcname: getviewsblock
@@ -138,6 +156,9 @@ class GopherPlusProtocol(GopherProtocol):
         self.wfile.write(self.config.get("protocols.gopherp.GopherPlusProtocol", "admin"))
         self.wfile.write("\r\n" + msg + "\r\n")
 
+    def groksabstract(self):
+        return 1
+
 class URLGopherPlus(GopherPlusProtocol):
     def getsupportedblocknames(self):
         return GopherPlusProtocol.getsupportedblocknames(self) + \
